<?php
namespace nulib\db\_private;

use nulib\db\IDatabase;
use nulib\php\func;

abstract class _migration {
  const MIGRATION = null;

  function __construct($migrations, string $channel="", ?IDatabase $db=null) {
    $this->db = $db;
    $this->channel = $channel;
    if ($migrations === null) $migrations = static::MIGRATION;
    if ($migrations === null) $migrations = [];
    elseif (is_string($migrations)) $migrations = [$migrations];
    elseif (is_callable($migrations)) $migrations = [$migrations];
    elseif (!is_array($migrations)) $migrations = [strval($migrations)];
    $this->migrations = $migrations;
  }

  protected ?IDatabase $db;

  protected string $channel;

  /** @var callable[]|string[]  */
  protected $migrations;

  abstract function setup(): void;
  abstract function beforeMigrate(string $key): bool;
  abstract function afterMigrate(string $key): void;

  function migrate(?IDatabase $db=null): void {
    $db = ($this->db ??= $db);
    $this->setup();
    foreach ($this->migrations as $key => $migration) {
      if (!$this->beforeMigrate($key)) continue;
      if (is_string($migration) || !func::is_callable($migration)) {
        $db->exec($migration);
      } else {
        func::with($migration)->bind($this, true)->invoke([$db, $key]);
      }
      $this->afterMigrate($key);
    }
  }
}
